@@ -12,7 +12,7 @@
  * @author     Martin Jansen <mj@php.net>
  * @copyright  1997-2009 The Authors
  * @license    http://opensource.org/licenses/bsd-license.php New BSD License
- * @version    CVS: $Id: Downloader.php 287109 2009-08-11 18:50:30Z dufuz $
+ * @version    CVS: $Id: Downloader.php 308691 2011-02-25 23:25:02Z dufuz $
  * @link       http://pear.php.net/package/PEAR
  * @since      File available since Release 1.3.0
  */
@@ -39,7 +39,7 @@ define('PEAR_INSTALLER_ERROR_NO_PREF_STATE', 2);
  * @author     Martin Jansen <mj@php.net>
  * @copyright  1997-2009 The Authors
  * @license    http://opensource.org/licenses/bsd-license.php New BSD License
- * @version    Release: 1.9.0
+ * @version    Release: 1.9.2
  * @link       http://pear.php.net/package/PEAR
  * @since      Class available since Release 1.3.0
  */
@@ -189,7 +189,8 @@ class PEAR_Downloader extends PEAR_Common
             require_once 'System.php';
         }
 
-        $tmp = System::mktemp(array('-d'));
+        $tmpdir = $this->config->get('temp_dir');
+        $tmp = System::mktemp("-d -t $tmpdir");
         $a   = $this->downloadHttp('http://' . $channel . '/channel.xml', $this->ui, $tmp, $callback, false);
         PEAR::popErrorHandling();
         if (PEAR::isError($a)) {
@@ -493,14 +494,13 @@ class PEAR_Downloader extends PEAR_Common
      */
     function analyzeDependencies(&$params, $force = false)
     {
-        $hasfailed = $failed = false;
         if (isset($this->_options['downloadonly'])) {
             return;
         }
 
         PEAR::staticPushErrorHandling(PEAR_ERROR_RETURN);
         $redo  = true;
-        $reset = false;
+        $reset = $hasfailed = $failed = false;
         while ($redo) {
             $redo = false;
             foreach ($params as $i => $param) {
@@ -553,7 +553,7 @@ class PEAR_Downloader extends PEAR_Common
                     }
 
                     $failed = false;
-                    if (isset($deps['required'])) {
+                    if (isset($deps['required']) && is_array($deps['required'])) {
                         foreach ($deps['required'] as $type => $dep) {
                             // note: Dependency2 will never return a PEAR_Error if ignore-errors
                             // is specified, so soft is needed to turn off logging
@@ -587,7 +587,7 @@ class PEAR_Downloader extends PEAR_Common
                             }
                         }
 
-                        if (isset($deps['optional'])) {
+                        if (isset($deps['optional']) && is_array($deps['optional'])) {
                             foreach ($deps['optional'] as $type => $dep) {
                                 if (!isset($dep[0])) {
                                     if (PEAR::isError($e =
@@ -698,6 +698,7 @@ class PEAR_Downloader extends PEAR_Common
                 }
             }
         }
+
         PEAR::staticPopErrorHandling();
         if ($hasfailed && (isset($this->_options['ignore-errors']) ||
               isset($this->_options['nodeps']))) {
@@ -718,6 +719,7 @@ class PEAR_Downloader extends PEAR_Common
         if (isset($this->_downloadDir)) {
             return $this->_downloadDir;
         }
+
         $downloaddir = $this->config->get('download_dir');
         if (empty($downloaddir) || (is_dir($downloaddir) && !is_writable($downloaddir))) {
             if  (is_dir($downloaddir) && !is_writable($downloaddir)) {
@@ -725,14 +727,17 @@ class PEAR_Downloader extends PEAR_Common
                     '" is not writeable.  Change download_dir config variable to ' .
                     'a writeable dir to avoid this warning');
             }
+
             if (!class_exists('System')) {
                 require_once 'System.php';
             }
+
             if (PEAR::isError($downloaddir = System::mktemp('-d'))) {
                 return $downloaddir;
             }
             $this->log(3, '+ tmp dir created at ' . $downloaddir);
         }
+
         if (!is_writable($downloaddir)) {
             if (PEAR::isError(System::mkdir(array('-p', $downloaddir))) ||
                   !is_writable($downloaddir)) {
@@ -741,6 +746,7 @@ class PEAR_Downloader extends PEAR_Common
                     'a writeable dir');
             }
         }
+
         return $this->_downloadDir = $downloaddir;
     }
 
@@ -771,27 +777,11 @@ class PEAR_Downloader extends PEAR_Common
         $this->_options = $options;
     }
 
-    // }}}
-    // {{{ setOptions()
     function getOptions()
     {
         return $this->_options;
     }
 
-    /**
-     * For simpler unit-testing
-     * @param PEAR_Config
-     * @param int
-     * @param string
-     */
-    function &getPackagefileObject(&$c, $d, $t = false)
-    {
-        if (!class_exists('PEAR_PackageFile')) {
-            require_once 'PEAR/PackageFile.php';
-        }
-        $a = &new PEAR_PackageFile($c, $d, $t);
-        return $a;
-    }
 
     /**
      * @param array output of {@link parsePackageName()}
@@ -1000,9 +990,20 @@ class PEAR_Downloader extends PEAR_Common
             }
             return $info;
         } elseif ($chan->supportsREST($this->config->get('preferred_mirror'))
-              && $base = $chan->getBaseURL('REST1.0', $this->config->get('preferred_mirror'))
+              &&
+                (
+                  ($base2 = $chan->getBaseURL('REST1.3', $this->config->get('preferred_mirror')))
+                    ||
+                  ($base = $chan->getBaseURL('REST1.0', $this->config->get('preferred_mirror')))
+                )
         ) {
-            $rest = &$this->config->getREST('1.0', $this->_options);
+            if ($base2) {
+                $base = $base2;
+                $rest = &$this->config->getREST('1.3', $this->_options);
+            } else {
+                $rest = &$this->config->getREST('1.0', $this->_options);
+            }
+
             $url = $rest->getDepDownloadURL($base, $xsdversion, $dep, $parr,
                     $state, $version, $chan->getName());
             if (PEAR::isError($url)) {
@@ -1595,7 +1596,7 @@ class PEAR_Downloader extends PEAR_Common
 
             if ($lastmodified === false || $lastmodified) {
                 $request  = "GET $url HTTP/1.1\r\n";
-                $request .= "Host: $host:$port\r\n";
+                $request .= "Host: $host\r\n";
             } else {
                 $request  = "GET $url HTTP/1.0\r\n";
                 $request .= "Host: $host\r\n";
@@ -1617,7 +1618,7 @@ class PEAR_Downloader extends PEAR_Common
 
             if ($lastmodified === false || $lastmodified) {
                 $request = "GET $path HTTP/1.1\r\n";
-                $request .= "Host: $host:$port\r\n";
+                $request .= "Host: $host\r\n";
             } else {
                 $request = "GET $path HTTP/1.0\r\n";
                 $request .= "Host: $host\r\n";
@@ -1638,7 +1639,7 @@ class PEAR_Downloader extends PEAR_Common
         }
 
         $request .= $ifmodifiedsince .
-            "User-Agent: PEAR/1.9.0/PHP/" . PHP_VERSION . "\r\n";
+            "User-Agent: PEAR/1.9.2/PHP/" . PHP_VERSION . "\r\n";
 
         if (isset($this)) { // only pass in authentication for non-static calls
             $username = $config->get('username', null, $channel);
@@ -1707,6 +1708,10 @@ class PEAR_Downloader extends PEAR_Common
         }
 
         $dest_file = $save_dir . DIRECTORY_SEPARATOR . $save_as;
+        if (is_link($dest_file)) {
+            return PEAR::raiseError('SECURITY ERROR: Will not write to ' . $dest_file . ' as it is symlinked to ' . readlink($dest_file) . ' - Possible symlink attack');
+        }
+
         if (!$wp = @fopen($dest_file, 'wb')) {
             fclose($fp);
             if ($callback) {
@@ -1758,5 +1763,4 @@ class PEAR_Downloader extends PEAR_Common
         }
         return $dest_file;
     }
-}
-// }}}
\ No newline at end of file
+}
\ No newline at end of file
